@@ -1,5 +1,3 @@
-# $Id: LanguageTool.py 14963 2005-12-16 13:28:53Z hannosch $
-
 from AccessControl import ClassSecurityInfo
 from Globals import InitializeClass
 from OFS.SimpleItem import SimpleItem
@@ -16,7 +14,6 @@ from Products.CMFCore.ActionInformation import ActionInformation
 from Products.CMFCore.ActionProviderBase import ActionProviderBase
 from Products.CMFCore.utils import UniqueObject, getToolByName
 from Products.PageTemplates.PageTemplateFile import PageTemplateFile
-from Acquisition import aq_base
 from ZPublisher import BeforeTraverse
 from ZPublisher.HTTPRequest import HTTPRequest
 
@@ -188,14 +185,44 @@ class LanguageTool(UniqueObject, ActionProviderBase, SimpleItem):
 
     security.declarePublic('getAvailableLanguages')
     def getAvailableLanguages(self):
-        """Returns the dictionary of available languages."""
-        langs = availablelanguages.languages.copy()
+        """Returns the dictionary of available languages.
+        The dict should have the form: {code : {native, english, flag}}.
+        """
+        langs = availablelanguages.getNativeLanguageNames()
         if self.use_combined_language_codes:
-            langs.update(availablelanguages.combined)
+            langs.update(availablelanguages.getCombinedLanguageNames())
         if self.local_available_langs.keys():
             langs.update(self.local_available_langs)
         return langs
 
+    security.declarePublic('listAvailableLanguageInformation')
+    def listAvailableLanguageInformation(self):
+        """Returns list of available languages."""
+        langs = self.getAvailableLanguageInformation()
+        new_langs = []
+        for lang in langs:
+            # add language-code to dict
+            langs[lang]['code'] = lang
+            # flatten outer dict to list to make it sortable
+            new_langs.append(langs[lang])
+        new_langs.sort(lambda x, y: cmp(x.get('native', x.get('english')), y.get('native', y.get('english'))))
+        return new_langs
+
+    security.declarePublic('getAvailableLanguageInformation')
+    def getAvailableLanguageInformation(self):
+        """Returns the dictionary of available languages."""
+        langs = availablelanguages.getLanguages()
+        if self.use_combined_language_codes:
+            langs.update(availablelanguages.getCombined())
+        if self.local_available_langs.keys():
+            langs.update(self.local_available_langs)
+        for lang in langs:
+            if lang in self.supported_langs:
+                langs[lang]['selected'] = True
+            else:
+                langs[lang]['selected'] = False
+        return langs
+
     security.declareProtected(View, 'getDefaultLanguage')
     def getDefaultLanguage(self):
         """Returns the default language."""
@@ -241,6 +268,14 @@ class LanguageTool(UniqueObject, ActionProviderBase, SimpleItem):
         """Returns the name for a language code."""
         return self.getAvailableLanguages().get(langCode, langCode)
 
+    security.declareProtected(View, 'getFlagForLanguageCode')
+    def getFlagForLanguageCode(self, langCode):
+        """Returns the name of the flag for a language code."""
+        info = self.getAvailableLanguageInformation().get(langCode, None)
+        if info is not None:
+            return info.get('flag', None)
+        return None
+
     security.declareProtected(ManagePortal, 'addSupportedLanguage')
     def addSupportedLanguage(self, langCode):
         """Registers a language code as supported."""
